@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Header: /Users/claude/fuzz/lib/Genezzo/Row/RCS/RSExpr.pm,v 7.1 2005/07/19 07:49:03 claude Exp claude $
+# $Header: /Users/claude/fuzz/lib/Genezzo/Row/RCS/RSExpr.pm,v 7.2 2005/11/26 02:10:29 claude Exp claude $
 #
 # copyright (c) 2005 Jeffrey I Cohen, all rights reserved, worldwide
 #
@@ -77,8 +77,10 @@ sub _init
     {
 #        greet $args{select_list};
         $self->{select_list} = $args{select_list};
+        return 0
+            unless (defined($args{alias}));
+        $self->{alias} = $args{alias};
     }
-
     return 1;
 }
 
@@ -230,6 +232,7 @@ sub SQLPrepare # get a DBI-style statement handle
     if (defined($self->{select_list}))
     {
         $args{select_list} = $self->{select_list};
+        $args{alias}       = $self->{alias};
     }
     $args{use_select_list} = defined($self->SelectList());
 
@@ -261,7 +264,10 @@ sub _init
     return 0
         unless (defined($args{rs}));
     my $rs = $args{rs};
-    $self->{sql_rs}   = $rs->SQLPrepare(@_);
+
+    my %nargs = @_;
+
+    $self->{sql_rs}   = $rs->SQLPrepare(%nargs);
     return 0
         unless (defined($self->{sql_rs}));
 
@@ -269,6 +275,9 @@ sub _init
     {
 #        greet $args{select_list};
         $self->{select_list} = $args{select_list};
+        return 0
+            unless (defined($args{alias}));
+        $self->{alias} = $args{alias};
     }
 
     $self->{rownum} = 0;
@@ -302,6 +311,8 @@ sub new
 
 } # end new
 
+# XXX XXX: where is SQLExecute?
+
 sub SQLFetch
 {
     my $self = shift;
@@ -326,6 +337,8 @@ sub SQLFetch
         if ($self->{use_select_list})
         {
             my $outarr = $vv;
+            my $alias = $self->{alias};
+            my $get_alias_col = {$alias => $outarr};
 
             for my $valex (@{$self->{select_list}})
             {
@@ -469,10 +482,12 @@ Genezzo::Row::RSExpr - Row Source Expression Evaluation
 
 various
 
-=head1 #TODO
+=head1 TODO
 
 =over 4
 
+=item SQLPrepare/SQLFetch: requires ALIAS argument, which doesn't make sense for rowsources like RSDual (see XEval).  "Alias" is only necessary to disambiguate named columns.
+
 =back
 
 =head1 AUTHOR
